/*
 * Brzozowski.h
 *
 *  Created on: 11. 1. 2014
 *      Author: tomas
 */

#ifndef BRZOZOWSKI_H_
#define BRZOZOWSKI_H_

#include <map>
#include <set>
#include <deque>

#include <automaton/State.h>
#include <AlibException.h>

#include "AbstractREtoFAConverter.h"

#include "RegExpDerivation.h"
#include "RegExpAlphabet.h"
#include "RegExpOptimize.h"

namespace conversions
{

/**
 * Converts regular expression to finite automata using Brzozowski algorithm (derivations of regular expressions).
 */
class Brzozowski : public AbstractREtoFAConverter
{
public:
    Brzozowski( const regexp::RegExp & re );
    ~Brzozowski( void );
    automaton::FSM convert( void );

private:
    class StateBuilder
    {
    public:
        StateBuilder( const std::set<regexp::RegExp> & Q );
        const automaton::State & getState( const regexp::RegExp & re ) const;

    private:
        std::string createNewName( void );
        std::map<const regexp::RegExp, automaton::State> m_states;
        unsigned int m_stateId;
    };

    struct Transition
    {
        const regexp::RegExp m_from, m_to;
        const alphabet::Symbol m_symbol;

        Transition( const regexp::RegExp & from, const alphabet::Symbol & symb, const regexp::RegExp & to );
        bool operator<( const Transition & other ) const;
    };

    std::set<Transition> m_transitions;
};

} /* namespace conversions */

#endif /* BRZOZOWSKI_H_ */
