/*
 * RegularEquationSolver.cpp
 *
 *  Created on: 4. 3. 2014
 *      Author: tomas
 */

#include "RegularEquationSolver.h"

using namespace alib;
using namespace alphabet;
using namespace regexp;

namespace conversions
{

RegularEquationSolver::RegularEquationSolver( void )
{

}

RegularEquationSolver::~RegularEquationSolver( void )
{

}

RegExp RegularEquationSolver::solve( const Symbol & solveFor )
{
    if( ! isInSet( solveFor, m_symbols ) )
        throw AlibException( "Trying to solve for symbol not present in equation system" );

    /*
     * Firstly, organize states by depth so we can output better looking
     * expressions. We need to solve equation system for automata initial state,
     * so lets start with the deepest ones and walk towards initial one.
     */
    symbolsByDepth( solveFor );

    RegExpElement* result = eliminate( );

    RegExp ret;
    ret.setRegExp( result );
    return ret;
}

void RegularEquationSolver::addSymbol( const Symbol & symb )
{
    m_symbols.insert( symb );

    for( const auto & s1 : m_symbols )
    {
        m_eqTransition[ symb ][ s1 ] = new RegExpEmpty( );
        if( s1 != symb )
            m_eqTransition[ s1 ][ symb ] = new RegExpEmpty( );
    }

    m_eqFinal[ symb ] = new RegExpEmpty( );
}

void RegularEquationSolver::addEquation( const Symbol & from, const Symbol & to, const RegExpElement* eq )
{
    RegExpOptimize opt;

    if( ! isInSet( from, m_symbols ) )
        throw AlibException( "Symbol '" + from.getSymbol( ) + "' not in equation system." );

    if( ! isInSet( to, m_symbols ) )
        throw AlibException( "Symbol '" + to.getSymbol( ) + "' not in equation system." );

    Alternation* alt = new Alternation( );
    alt->getElements( ).push_back( eq->clone( ) );
    alt->getElements( ).push_back( m_eqTransition[ from ][ to ] );
    m_eqTransition[ from ][ to ] = opt.optimize( alt );
    delete eq;
    delete alt;
}

void RegularEquationSolver::addEquation( const Symbol & from, const RegExpElement* eq )
{
    RegExpOptimize opt;

    if( ! isInSet( from, m_symbols ) )
        throw AlibException( "Symbol '" + from.getSymbol() + "' not in equation system." );

    Alternation* alt = new Alternation( );
    alt->getElements( ).push_back( eq->clone( ) );
    alt->getElements( ).push_back( m_eqFinal[ from ] );
    m_eqFinal[ from ] = opt.optimize( alt );
    delete eq;
    delete alt;
}

void RegularEquationSolver::symbolsByDepth( const Symbol & solveFor )
{
    map<Symbol, bool> visited;
    queue<Symbol> queue;

    for( const auto & symbol : m_symbols )
        visited[ symbol ] = false;

    visited[ solveFor ] = true;
    queue.push( solveFor );


    while( ! queue.empty( ) )
    {
        Symbol s = queue.front( );
        queue.pop( );

        m_symbolsByDepth.push_back( s );

        auto map_s = m_eqTransition.find( s );
        for( auto it = map_s->second.begin( ); it != map_s->second.end( ); it ++ )
        {
            // neighbouring if regexp between s and kv.first is not \0
            if( dynamic_cast<RegExpEmpty*>( it->second ) == NULL && visited [ it->first ] == false )
            {
                visited[ it->first ] = true;
                queue.push( it->first );
            }
        }
    }
}


} /* namespace conversions */
