#include "GrammarPrinter.h"

using namespace std;
using namespace grammar;

GrammarPrinter::GrammarPrinter( std::ostream& out ) : m_Out( out )
{

}

void GrammarPrinter::print( const grammar::Grammar & grammar )
{
    // group by left side
    map< list<Symbol>, set<Rule> > group;
    for( const auto & r : grammar.getRules( ) )
    {
        if( group.find( r.getLeftSide( ) ) != group.end( ) )
            group[ r.getLeftSide( ) ].insert( r );
        else
            group.insert( std::pair< list<Symbol>, set<Rule> >( r.getLeftSide( ), set<Rule>( ) = { r } ) );
    }

    m_Out << "G = (N,T,P,S)" << endl;

    auto last = grammar.getNonTerminalSymbols( ).end( );
    last --;
    m_Out << "N = {";
    for( auto it = grammar.getNonTerminalSymbols( ).begin( ) ; it != grammar.getNonTerminalSymbols( ).end( ) ; it++ )
        m_Out << it->getSymbol( ) << ( it != last ? ", " : "" );
    m_Out << "}" << endl;

    last = grammar.getTerminalSymbols( ).end( );
    last --;
    m_Out << "T = {";
    for( auto it = grammar.getTerminalSymbols( ).begin( ) ; it != grammar.getTerminalSymbols( ).end( ) ; it++ )
        m_Out << it->getSymbol( ) << ( it != last ? ", " : "" );
    m_Out << "}" << endl;

    m_Out << "P = {" << endl;
    for( const auto & g : group )
        for( const auto & r : g.second )
            m_Out << "\t" << r.toString( ) << endl;
    m_Out << "}" << endl;

    m_Out << "S = " << grammar.getStartSymbol( ).getSymbol( ) << endl;
}

