#ifndef RHDPDADETERKMINIZATIONSTRUCTS_H_
#define RHDPDADETERKMINIZATIONSTRUCTS_H_

#include <set>

using namespace automaton;

namespace determinization {

struct StateSymbolPair {
    State state;
    Symbol symbol;

    StateSymbolPair(const State& state, const Symbol& symbol)
            : state(state),
              symbol(symbol) {}

    bool operator < (const StateSymbolPair& other) const {
        return (state < other.state) || ((state == other.state) && (symbol < other.symbol));
    }
    
    bool operator == (const StateSymbolPair& other) const {
        return (state == other.state) && (symbol == other.symbol);
    }
    
    bool operator != (const StateSymbolPair& other) const {
        return (state != other.state) || (symbol != other.symbol);
    }
};

struct PairOfPairs {
    StateSymbolPair pair1;
    StateSymbolPair pair2;
    
    PairOfPairs(const StateSymbolPair& pair1, const StateSymbolPair& pair2)
            : pair1(pair1),
              pair2(pair2) {}
    
    bool operator < (const PairOfPairs& other) const {
        return (pair1 < other.pair1) || ((pair1 == other.pair1) && (pair2 < other.pair2));
    }
    
    bool operator == (const PairOfPairs& other) const {
        return (pair1 == other.pair1) && (pair2 == other.pair2);
    }
    
    bool operator != (const PairOfPairs& other) const {
        return (pair1 != other.pair1) || (pair2 != other.pair2);
    }
};


struct SComponent {
    set<PairOfPairs> pairsOfPairs;
    SComponent() {}
    SComponent(const set<PairOfPairs>& pairsOfPairs)
            : pairsOfPairs(pairsOfPairs) {}
};


struct RComponent {
    set<StateSymbolPair> pairs;
    RComponent() {}
    RComponent(const set<StateSymbolPair>& pairs)
            : pairs(pairs) {}
};


struct StateData {
    State state;
    SComponent s;
    RComponent r;
    bool isMarked;
    
    StateData(const State& state, const SComponent& s, const RComponent& r)
            : state(state),
              s(s),
              r(r),
              isMarked(false) {}
};


struct PartialStackData {
    SComponent s;
    RComponent r;
    
    PartialStackData(const SComponent& s, const RComponent& r)
            : s(s),
              r(r) {}
};

} /* namespace determinization */
#endif /* RHDPDADETERKMINIZATIONSTRUCTS_H_ */