/*
 * StringFactory.h
 *
 *  Created on: Jan 1, 2014
 *      Author: Jan Travnicek
 */

#ifndef STRING_FACTORY_H_
#define STRING_FACTORY_H_

#include <list>

#include "sax/Token.h"
#include "string/String.h"

namespace notions {
namespace string {

/**
 * String builder.
 */
class StringFactory {
public:
	/**
	 * Parses the XML in file and returns the String.
	 * @param filename path to the file
	 * @return String
	 */
	static String fromFile(const std::string& filename);

	/**
	 * Parses the XML and returns the String.
	 * @param str string containing the XML
	 * @return String
	 */
	static String fromString(const std::string& str);
	
	/**
	 * Parses the XML from stdin and returns the String.
	 * @return String
	 */
	static String fromStdin();
	
	/**
	 * Parses the XML from stream and returns the String.
	 * @return String
	 */
	static String fromStream(std::istream& in);
	
protected:
	/**
	 * Parses the String from list of tokens.
	 * @param tokens XML represented as list of tokens
	 * @return parsed String
	 */
	static String parse(std::list<sax::Token> tokens);
};

} /* namespace string */
} /* namespace notions */

#endif /* STRING_FACTORY_H_ */
