/*
 * UnknownAutomaton.cpp
 *
 *  Created on: Oct 12, 2013
 *      Author: martin
 */

#include "UnknownAutomaton.h"

#include "AutomatonPrinter.h"
#include "exception/AutomatonException.h"

namespace automaton {

UnknownAutomaton::UnknownAutomaton() :
		blankSymbol("") {

}

UnknownAutomaton::~UnknownAutomaton() {
}

void UnknownAutomaton::removeState(const State& state) {
	if (!states.erase(state))
		throw AutomatonException("State \"" + state.getName() + "\" doesn't exist.");
}

void UnknownAutomaton::removeInputSymbol(const Symbol& symbol) {
	if (!inputAlphabet.erase(symbol))
		throw AutomatonException("Input symbol \"" + symbol.getSymbol() + "\" doesn't exist.");
}

const set<Symbol>& UnknownAutomaton::getStackAlphabet() const {
	return stackAlphabet;
}

void UnknownAutomaton::addStackSymbol(const Symbol& symbol) {
	if (!stackAlphabet.insert(symbol).second) {
		throw AutomatonException("Stack symbol \"" + symbol.getSymbol() + "\" already exists.");
	}
}

void UnknownAutomaton::removeStackSymbol(const Symbol& symbol) {
	if (!stackAlphabet.erase(symbol))
		throw AutomatonException("Stack symbol \"" + symbol.getSymbol() + "\" doesn't exist.");
}

const list<Symbol>& UnknownAutomaton::getStartSymbols() const {
	return startSymbols;
}

void UnknownAutomaton::setStartSymbols(const list<Symbol>& newSymbols) {
	this->startSymbols = newSymbols;
}

const set<Symbol>& UnknownAutomaton::getTapeAlphabet() const {
	return tapeAlphabet;
}

void UnknownAutomaton::addTapeSymbol(const Symbol& symbol) {
	if (!tapeAlphabet.insert(symbol).second) {
		throw AutomatonException("Tape symbol \"" + symbol.getSymbol() + "\" already exists.");
	}
}

void UnknownAutomaton::removeTapeSymbol(const Symbol& symbol) {
	int removed = tapeAlphabet.erase(symbol);
	if (!removed) {
		throw AutomatonException("Tape symbol \"" + symbol.getSymbol() + "\" doesn't exist.");
	}
}

const Symbol& UnknownAutomaton::getBlankSymbol() const {
	return blankSymbol;
}

void UnknownAutomaton::setBlankSymbol(const Symbol& symbol) {
	blankSymbol = symbol;
}

const set<UnknownTransition>& UnknownAutomaton::getTransitions() const {
	return transitions;
}

void UnknownAutomaton::addTransition(const UnknownTransition& transition) {
	if (!transitions.insert(transition).second) {
		throw AutomatonException("Transition already exists.");
	}
}

void UnknownAutomaton::removeTransition(const UnknownTransition& transition) {
	if (!transitions.erase(transition)) {
		throw AutomatonException("Transition doesn't exist.");
	}
}

void UnknownAutomaton::toXML(ostream& out) const {
	AutomatonPrinter::toXML(*this, out);
}

} /* namespace automaton */
