/*
 * ContextSensitiveGrammar.cpp
 *
 *  Created on: Nov 17, 2013
 *      Author: martin
 */

#include "ContextSensitiveGrammar.h"

namespace grammar {

ContextSensitiveGrammar::ContextSensitiveGrammar() :
		Grammar() {
}

ContextSensitiveGrammar::ContextSensitiveGrammar(const set<Symbol>& nonTerminalSymbols,
		const set<Symbol>& terminalSymbols, const Symbol& startSymbol) :
		Grammar(nonTerminalSymbols, terminalSymbols, startSymbol) {
}

bool ContextSensitiveGrammar::isValidRule(const Rule& rule) const {

	//left side cannot be longer than right side
	if (rule.getLeftSide().size() > rule.getRightSide().size()) {
		return false;
	}

	/*
	 * Finds nonterminal symbol in the left side of the rule and tries to match the symbols before nonterminal (prefix) to the beginning
	 * of the right side of the rule and symbols after the nonterminal (suffix) to the end of the right side of the rule.
	 */
	for (list<Symbol>::const_iterator symbol = rule.getLeftSide().begin(); symbol != rule.getLeftSide().end();
			symbol++) {
		if (nonTerminalSymbols.find(*symbol) != nonTerminalSymbols.end()) {
			if (startsWith(rule.getLeftSide().begin(), symbol, rule.getRightSide().begin())
					&& endsWith(symbol, rule.getLeftSide().end(), rule.getRightSide().end())) {
				return true;
			}
		}
	}

	return false;
}

bool ContextSensitiveGrammar::startsWith(list<Symbol>::const_iterator prefixStart,
		list<Symbol>::const_iterator prefixEnd, list<Symbol>::const_iterator rightSideStart) const {
	while (prefixStart != prefixEnd) {
		if (*prefixStart != *rightSideStart) {
			return false;
		}
		prefixStart++;
		rightSideStart++;
	}
	return true;
}

bool ContextSensitiveGrammar::endsWith(list<Symbol>::const_iterator suffixStart, list<Symbol>::const_iterator suffixEnd,
		list<Symbol>::const_iterator rightSideEnd) const {
	suffixEnd--;
	rightSideEnd--;

	while (suffixStart != suffixEnd) {

		if (*suffixEnd != *rightSideEnd) {
			return false;
		}
		suffixEnd--;
		rightSideEnd--;
	}
	return true;
}

} /* namespace grammar */

