/*
 * GrammarPrinter.cpp
 *
 *  Created on: Nov 5, 2013
 *      Author: martin
 */

#include "GrammarPrinter.h"

namespace grammar {

const string GrammarPrinter::INDENTATION = "\t";

void GrammarPrinter::toXML(const Grammar& grammar, ostream& out) {
	out << "<grammar>\n";
	printNonTerminalSymbols(grammar, out, INDENTATION);
	printTerminalSymbols(grammar, out, INDENTATION);
	printRules(grammar, out, INDENTATION);
	printSymbol(grammar.getStartSymbol(), out, "startSymbol", INDENTATION);
	out << "</grammar>\n";
}

void GrammarPrinter::printNonTerminalSymbols(const Grammar& grammar, ostream& out, string prefix) {
	out << prefix << "<nonTerminalSymbols>\n";
	for (auto &symbol : grammar.getNonTerminalSymbols()) {
		printSymbol(symbol, out, "symbol", prefix + INDENTATION);
	}
	out << prefix << "</nonTerminalSymbols>\n";
}

void GrammarPrinter::printTerminalSymbols(const Grammar& grammar, ostream& out, string prefix) {
	out << prefix << "<terminalSymbols>\n";
	for (auto& symbol : grammar.getTerminalSymbols()) {
		printSymbol(symbol, out, "symbol", prefix + INDENTATION);
	}
	out << prefix << "</terminalSymbols>\n";
}

void GrammarPrinter::printRules(const Grammar& grammar, ostream& out, string prefix) {
	out << prefix << "<rules>\n";
	for (auto& rule : grammar.getRules()) {
		printRule(rule, out, prefix + INDENTATION);
	}
	out << prefix << "</rules>\n";
}

void GrammarPrinter::printRule(const Rule& rule, ostream& out, string prefix) {
	out << prefix << "<rule>\n";
	out << prefix << INDENTATION << "<leftSide>\n";
	for (auto& symbol : rule.getLeftSide()) {
		printSymbol(symbol, out, "symbol", prefix + INDENTATION + INDENTATION);
	}
	out << prefix << INDENTATION << "</leftSide>\n";
	out << prefix << INDENTATION << "<rightSide>\n";
	for (auto& symbol : rule.getRightSide()) {
		printSymbol(symbol, out, "symbol", prefix + INDENTATION + INDENTATION);
	}
	out << prefix << INDENTATION << "</rightSide>\n";
	out << prefix << "</rule>\n";
}

void GrammarPrinter::printSymbol(const Symbol& symbol, ostream& out, string tagName, string prefix) {
	out << prefix << "<" << tagName << ">";
	out << symbol.getSymbol();
	out << "</" << tagName << ">\n";
}

} /* namespace grammar */
