/*
 * Alternation.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#include "Alternation.h"

namespace regexp {

Alternation::Alternation() {
}

Alternation::Alternation(const Alternation& other) {
	for (auto element : other.elements) {
		elements.push_back(element->clone());
	}
}

Alternation& Alternation::operator =(const Alternation& other) {
	if (this == &other) {
		return *this;
	}

	for (auto element : elements) {
		delete element;
	}
	elements.clear();

	for (auto element : other.elements) {
		elements.push_back(element->clone());
	}

	return *this;
}

Alternation::~Alternation() {
	for (auto element : elements) {
		delete element;
	}
	elements.clear();
}

list<RegExpElement*>& Alternation::getElements() {
	return elements;
}

const list<RegExpElement*>& Alternation::getElements() const {
	return elements;
}

RegExpElement* Alternation::clone() const {
	return new Alternation(*this);
}

bool Alternation::operator<(const RegExpElement& other) const {
	return other > *this;
}

bool Alternation::operator==(const RegExpElement& other) const {
	return other == *this;
}

bool Alternation::operator>(const RegExpElement& other) const {
	return other < *this;
}


bool Alternation::operator<(const Concatenation&) const {
	return true;
}

bool Alternation::operator<(const Alternation& other) const {
	int thisSize = this->elements.size();
	int otherSize = other.elements.size();
	if(thisSize < otherSize) return true;
	if(thisSize > otherSize) return false;

	auto thisIter = this->elements.begin();
	auto otherIter = other.elements.begin();
	for(; thisIter != this->elements.end(); thisIter++, otherIter++) {
	  if(**thisIter < **otherIter) return true;
	}

	return false;
}

bool Alternation::operator==(const Alternation& other) const {
	if(this->elements.size() != other.elements.size()) return false;

	auto thisIter = this->elements.begin();
	auto otherIter = other.elements.begin();
	for(; thisIter != this->elements.end(); thisIter++, otherIter++) {
	  if(**thisIter != **otherIter) return false;
	}

	return true;
}

bool Alternation::containsEmptyString() const {
	for(const auto& e : getElements())
		if(e->containsEmptyString())
			return true;

	return false;
}

} /* namespace regexp */

