/*
 * Iteration.h
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#ifndef ITERATION_H_
#define ITERATION_H_

#include <list>
#include "RegExpElement.h"

namespace regexp {

using namespace std;

/**
 * Represents iteration operator in the regular expression. Contains one RegExpElement
 * as operand.
 */
class Iteration: public RegExpElement {
private:
	RegExpElement* element;
public:
	Iteration();
	Iteration(const Iteration& other);
	Iteration& operator =(const Iteration& other);
	~Iteration();

	/**
	 * @return element to iterate
	 */
	RegExpElement* getElement();

	/**
	 * @return element to iterate
	 */
	const RegExpElement* getElement() const;

	/**
	 * Sets the element to iterate. Doesn't perform the copy, just stores the pointer!
	 * @param element RegExpElement to set
	 */
	void setElement(RegExpElement* element);

	/**
	 * @copydoc RegExpElement::clone() const
	 */
	RegExpElement* clone() const;

	virtual bool operator<(const RegExpElement&) const;
	virtual bool operator==(const RegExpElement&) const;
	virtual bool operator>(const RegExpElement&) const;

	virtual bool operator<(const Concatenation&) const;
	virtual bool operator<(const Alternation&) const;
	virtual bool operator<(const Iteration&) const;
	virtual bool operator==(const Iteration&) const;

	/**
	 * @copydoc RegExpElement::containsEmptyString() const
	 */
	bool containsEmptyString() const;
};

} /* namespace regexp */
#endif /* ITERATION_H_ */
