/*
 * String.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef STRING_H_
#define STRING_H_

#include <iostream>
#include <set>
#include <list>

#include "../alphabet/Symbol.h"

namespace notions {
namespace string {

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of StringElement.
 */
class String {
private:
	std::set<alphabet::Symbol> alphabet;
	std::list<alphabet::Symbol> symbols;

public:
	String();
	
	/**
	 * Adds input symbol to the alphabet.
	 * @param symbol Symbol to add
	 * @throws AutomatonException when symbol already exists
	 */
	void addAlphabetSymbol(const alphabet::Symbol& symbol);

	/**
	 * Removes input symbol from the the alphabet.
	 * @param symbol Symbol to remove
	 * @throws AutomatonException when symbol is not present in input alphabet
	 * or when symbol is part of the transition
	 */
	void removeAlphabetSymbol(const alphabet::Symbol& symbol);
	
	/**
	 * @return the input alphabet
	 */
	const std::set<alphabet::Symbol>& getAlphabet() const;

	/**
	 * @return Root node of the regular expression tree
	 */
	std::list<alphabet::Symbol>& getContent();

	/**
	 * @return Root node of the regular expression tree
	 */
	const std::list<alphabet::Symbol>& getContent() const;

	/**
	 * @return true if regexp represents empty language
	 */
	bool isEmpty() const;

	/**
	 * Prints XML representation of the String to the output stream.
	 * @param out output stream to which print the String
	 */
	void toXML(std::ostream& out);

	/**
	 * Prints XML representation of the String to the output stream.
	 * @param out output stream to which print the String
	 * @param regexp String to print
	 */
	friend std::ostream& operator<<(std::ostream& out, String& regexp);

};

} /* namespace string */
} /* namespace notions */
#endif /* STRING_H_ */
