/*
 * RegExpParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "StringParser.h"
#include "../sax/ParserException.h"

namespace notions {
namespace string {

String StringParser::parse(std::list<sax::Token>& input) {
	popToken(input, sax::Token::START_ELEMENT, "string");
	String string;
	parseAlphabet(input, string);
	parseContent(input, string);
	popToken(input, sax::Token::END_ELEMENT, "string");

	return string;
}

void StringParser::parseContent(std::list<sax::Token>& input, String& string) {
	popToken(input, sax::Token::START_ELEMENT, "content");
	
	while (isToken(input, sax::Token::START_ELEMENT, "symbol")) {
		popToken(input, sax::Token::START_ELEMENT, "symbol");
		
		if (input.front().getType() == sax::Token::CHARACTER) {
			alphabet::Symbol symbol(input.front().getData());
			input.pop_front();
			string.getContent().push_back(symbol);
		} else {
			throw sax::ParserException(sax::Token("", sax::Token::CHARACTER), input.front());
		}
		
		popToken(input, sax::Token::END_ELEMENT, "symbol");
	}
	
	popToken(input, sax::Token::END_ELEMENT, "content");
}

void StringParser::parseAlphabet(std::list<sax::Token>& input, String& string) {
	popToken(input, sax::Token::START_ELEMENT, "alphabet");
	
	while (isToken(input, sax::Token::START_ELEMENT, "symbol")) {
		popToken(input, sax::Token::START_ELEMENT, "symbol");
		
		if (input.front().getType() == sax::Token::CHARACTER) {
			alphabet::Symbol symbol(input.front().getData());
			input.pop_front();
			string.addAlphabetSymbol(symbol);
		} else {
			throw sax::ParserException(sax::Token("", sax::Token::CHARACTER), input.front());
		}
		
		popToken(input, sax::Token::END_ELEMENT, "symbol");
	}
	
	popToken(input, sax::Token::END_ELEMENT, "alphabet");
}

bool StringParser::isToken(std::list<sax::Token>& input, sax::Token::TokenType type, std::string data) {
	return input.front().getType() == type && input.front().getData() == data;
}

void StringParser::popToken(std::list<sax::Token>& input, sax::Token::TokenType type, std::string data) {
	if (isToken(input, type, data)) {
		input.pop_front();
	} else {
		throw sax::ParserException(sax::Token(data, type), input.front());
	}
}

} /* namespace regexp */
} /* namespace notions */
